%% AC fold demo
% $p_1 u_{xx}+\lambda u+u^3-p_3 u^5=0$ with Neumann b.c. on [-1,1]x[-0.9,0.9]
%%
% Use the following as command templates and run cell-by-cell 
close all; format compact; keep pphome; 
p=[]; nx=20; p=acfold_init(p,nx); 
p=findbif(p,4); % find bifpoints from trivial branch: 
%p.sw.bifloc=4; p=cont(p,10);
%% First two bifurcating branches, second without fold detection
p=swibra('p','bpt1','q',-0.1); 
p.nc.dsmin=1e-5; p.sw.foldcheck=1; p.nc.amod=0; p.file.smod=2; p=cont(p);
%%
p=swibra('p','bpt2','r',-0.1); p.nc.amod=10; p=cont(p,15);
%% Plot first bifurcation diagram
figure(3);clf; cmp=0; plotbraf('p',3,cmp,'labi',3); 
plotbraf('q',3,cmp,'lab',10,'cl', 'b','labi',4);
plotbraf('r',3,cmp,'labi',3, 'cl', 'r');
xlabel('\lambda');ylabel('||u||_2');
%% black-and-white plotting of branches with different line-types 
figure(3);clf; cmp=0; plotbraf('p',3,cmp,'cl','k'); 
plotbraf('q',3,cmp,'labi',2,'cl', 'k','tyst','--','tyun','-.');
plotbraf('r',3,cmp,'lab',10, 'cl', 'k','tyst','--','tyun',':');
xlabel('\lambda');ylabel('||u||_2');
%% Plotting from file 
plotsolf('q','pt10',4,1,2); plotsolf('r','pt7',5,1,2); 
%% Switch to continuation in diffusion constant 
clf(2); p=swiparf('q','pt10','w',2); 
p.sol.xi=1e-6; % small xi since problem quite sensitive in diff coeff. 
p.nc.lammin=0.1; p.sol.ds=-0.01; p=cont(p); % continue
%% Some a posteriori error checks
p=loadp('q','pt12'); err=errcheck(p); fprintf('error estimate=%g\n',err);
%% Fold continuation from first non-trivial branch (alternatively run next cell!) 
p=spcontini('q','fpt1',3,'qf');   % init fold continuation with par 3 new primary par
p.plot.bpcmp=p.nc.ilam(2); clf(2); % use this new parameter for plotting
p.nc.tol=1e-5;                  % increase tolerance as typically required for fold cont.
p.sol.ds=1e-3;                   % new stepsize in new primary parameter
p.sw.spjac=1;  p=cont(p);
%% Fold localization from non-fold point, and continuation of fold 
p=spcontini('q','pt4',3,'qf');   % init fold continuation with par 3 new primary par
p.sw.spcont=2; % reset value to spcont; first nloops then yield localization of fold! 
p.plot.bpcmp=p.nc.ilam(2); clf(2); % rest as above 
p.nc.tol=1e-5; p.sol.ds=1e-3; p.sw.spjac=1;  p=cont(p);
%% Switch back to regular continuation from one of the fold points
p=spcontexit('qf','pt10','q1');
p.nc.tol=1e-8; p.sol.ds=1e-3; p.plot.bpcmp=0;
p.sw.spcalc=1; clf(2); p=cont(p); clf(3); plotbra(p,3,0,'lab',1);
%% Branch point continuation on trivial solution
p=spcontini('p','bpt1',2,'pb');    % init branch point continuation in par 3
p.plot.bpcmp=p.nc.ilam(2); clf(2); % use this new parameter for plotting
p.nc.tol=1e-5;                  % increase tolerance as typically required for fold cont.
p.sol.ds=1e-3;                   % new stepsize in new primary parameter
p.sw.spjac=0; p.nc.nsteps=10; p.nc.amod=0; %p=cont(p);
%% Branch point localization on trivial branch, and subsequent continuation
p=spcontini('p','pt1',2,'pb');    % init branch point continuation in par 3
p.sw.spcont=1;
p.plot.bpcmp=p.nc.ilam(2); figure(1); clf; % rest as above 
p.nc.tol=1e-5; p.sol.ds=1e-3; p.sw.spjac=1; p.nc.nsteps=10; p.nc.amod=0; p=cont(p);
%% Exit branch point continuation and get tangent
p=spcontexit('pb','pt10','q2'); p.tau=getinitau(p); clf(2);
%% Switch to nontrivial branch and continue
p=swibra(p,0.05,'q2'); p.nc.ilam=1; p.plot.bpcmp=0;
p.nc.dsmin=1e-5; p.nc.amod=5; p=cont(p);
%% Illustrate time integration, tints vs tint 
% See schnakenberg section of manual for explanation
p=loadp('q','pt5','tt');
p.u(1:p.np)=p.u(1:p.np)+0.5*rand(p.np,1); plotsol(p,1,1,3);p0=p;
%% With saving and time-series output 
p=p0; t=0; ts=[]; dt=0.1; nt=1000; nc=0; pmod=nt/5; smod=nt; 
tic; [p,t,ts,nc]=tintx(p,t,ts,dt,nt,nc,pmod,smod); toc % slow version 
par=p.u(p.nu+1:end); K=par(2)*p.mat.K; % put diff constant directly into K
% tic; [p,t,ts,nc]=tintxs(p,t,ts,dt,nt,nc,pmod,smod,@acfold_nf,K); toc
%% Time integration with simple syntax implementation for comparison
p=p0;par=p.u(p.nu+1:end);K=par(2)*p.mat.K;
tic;p=tints(p,dt,nt,pmod,@acfold_nf,K);toc 
%% Repeat first two cells with saving meshes to ./meshes/ and symmetric poimesh 
close all; clear all; p=[]; nx=10; p=acfold_init(p,nx,1); % 1 for p.mesh.sympoi
p=setfn(p,'pnm'); % nm for 'no mesh' 
p.file.msave=0; p=savemesh(p); p=findbif(p,2); 
%% First two bifurcating branches, second without fold detection
p=swibra('pnm','bpt1','qnm',-0.2); p.nc.dsmin=1e-5; p.sw.foldcheck=1; p.nc.amod=10; p=cont(p,15);
p=swibra('pnm','bpt2','rnm',-0.1); p.nc.amod=10; p=cont(p,15);
%% Links to source code
% * <acfold_G.html acfold_G.m>, <acfold_Gjac.html acfold_Gjac.m> - pde and jacobians in generic format
% * <acfold_sG.html acfold_G.m>, <acfold_nf.html acfold_nf.m>, <acfold_sGjac.html acfold_Gjac.m> - same in simple format (see manual)
% * <acfold_spjac.html acfold_spjac.m> - extra jacobian for fold/branch point ('spectral') continuation
% * <acfold_qf.html acfold_qf.m>, <acfold_qfder.html acfold_qfder.m> - auxiliary equation and jacobian
% * <acfold_init.html acfold_init.m> - initialization routine with domain, boundary conditions, initial mesh, etc.