%%
% MSWIBRA: 'multiple' zero eigenvalue version of swibra
%
% algorithm:  calculate eigenvectors to eigenvalues mu with abs(mu)<p.nc.mu2 
% orthogonalize via Gram-Schmidt
% divide every tangent by the maximum of its first component, 
% store these tangents columnwise in p.sol.tau and the first one in p.tau,
% plot the n tangents in figures 21, 22,...,20+n 
function q=mswibra(dir,fname,varargin)
if(ischar(dir)) p=loadp(dir,fname); 
else p=dir; if nargin>1; varargin{2}=fname; end; fname=inputname(1);
end
if p.sol.ptype==3; fprintf('\n Hopf-point, use hoswibra\n'); q=[]; return; end
%if(p.sol.ptype~=1 && p.sw.spcont~=1) fprintf('\nInitialization error: not a branch point.\n'); end
u=p.u; lam=getlam(p); tau=p.tau; xi=p.sol.xi; lvar=length(varargin);
pstyle=p.plot.pstyle; del=1e-3; % don't choose del too small!
if lvar>1; p.sol.ds=varargin{2}; gotds=1; lvar=lvar-1; 
    if lvar>1; pstyle=varargin{3}; end
else p.sol.ds=p.nc.dsmax/10; gotds=0; end 
r=resi(p,u); [Gu,Glam]=getder(p,u,r); % residual and jacobians 
% now calc al0, al1, phi0
vs=size(Gu,1); u0d=tau(1:p.nu+p.nc.nq); al0=tau(p.nu+p.nc.nq+1); 
if al0==0 fprintf('Degenerate branch point with d/ds lambda=0');return; end
if p.sw.eigsstart==1; p.sw.evopts.v0=ones(vs,1)/vs; % set eigs startvector
else p.sw.evopts.v0=(rand(vs,1)-1)/vs; 
end 
zz=0; p.sol.muv(1:6)
for i=1:length(p.sol.muv)
   if abs(real(p.sol.muv(i)))<p.nc.mu2 && abs(imag(p.sol.muv(i)))<p.nc.mu2; zz=zz+1;end;
end; 
if zz==0; fprintf('no zero evals\n'); q=0; return; end
fprintf(['lam=' num2str(lam), ', ' num2str(zz),' real zero eigenvalues: ' num2str(p.sol.muv(1:zz)) '\n']);
[phi1v,mu]=eigs(Gu,p.mat.M,zz,0,p.sw.evopts);
mtau=[];
for j=1:zz
  phi1=real(phi1v(:,j));
  [psi1v,mu]=eigs(Gu',p.mat.M,j,0,p.sw.evopts); psi1=psi1v(:,j);psi1=psi1/(phi1'*psi1);
  al1=psi1'*u0d;  phi0=(u0d-al1*phi1)/al0; 
  rp=resi(p,au2u(p,u2au(p,u)+del*phi1)); % residual and jacobian at u+del*phi1 
  [Gup,Glamp]=getder(p,au2u(p,u2au(p,u)+del*phi1),rp); Gud=Gup-Gu; 
  if any(Gud) 
   Glamd=Glamp-Glam; a1=psi1'*(Gud*phi1)/del; 
   b1=psi1'*(Gud*phi0+Glamd)/del; al1b=-(a1*al1/al0+2*b1); 
   fprintf('al0=%g, a1=%g, b1=%g, al1b=%g\n',al0,a1,b1,al1b); 
   if al1b==0 printf('No distinct branch to switch to.');return; end
   tau1=[al1b*phi1+a1*phi0; a1];  
  else; tau1=[phi1; 0];fprintf('trivial swibra\n'); % trivial branch 
  end
  if(p.sw.inter>1); p.sol.xi=asknu('xi',xi); 
    if ~gotds; p.sol.ds=asknu('ds',p.sol.ds); end 
  end
  mtau=[mtau,tau1];
end %j=1:zz
mtau=gramschmidt(p,mtau); p.sol.mtau=mtau; p.tau=mtau(:,1);
bs=p.branch(:,size(p.branch,2)); p=resetc(p); % set counters (note branch set below)
p.sol.ptype=-2; bs(1)=0; bs(2)=p.sol.ptype; % put last of p on new branch
if(~isempty(varargin)); [p,ok]=setfn(p,varargin{1}); if ok~=1; q=p; return; end
else fprintf('warning: problem directory unchanged.\n'); end
p.branch=bs; p.sol.deta=0; p.file.count=0; 
p.fuha.savefu(p); p.file.count=1; q=p;
% plot 
set(0,'Units','pixels'); scnsize = get(0,'ScreenSize'); 
scnw=scnsize(3); scnh=scnsize(4);sz=scnh/3-scnh*9.5/100;
u0=p.u; 
for i=1:zz
 p.u(1:p.nu)=mtau(1:p.nu,i);
 if mod(i,3)==1; pheight=2*scnh/3; end; if mod(i,3)==2; pheight=sz+115; end; 
 if mod(i,3)==0;pheight=0; end;
 pw=ceil(i/3); pw=pw+1; %shift figure to the left
 set(figure(20+i),'Position',[scnw-pw*sz pheight sz sz]);
 plotsol(p,20+i,p.plot.pcmp,pstyle);
 %set(figure(30+i),'Position',[scnw-2*pw*sz pheight sz sz]);
 %fouplot(p,30+i,1,[4 3]);
end
p.u(1:p.nu)=u0(1:p.nu);
