%%
% SWIBRA: attempts branch switching step for p-struct  
% and prints diagnostic data for branch switch computation
%
%  q=swibra(p)
%  q=swibra(p,ds)                     - use initial stepsize ds
%  q=swibra(dir,fname)                - load p from dir/fname
%  q=swibra(dir,fname,dir)            - set problem name to dir
%  q=swibra(dir,fname,dir,ds)         - also set initial stepsize to ds
%  q=swibra(dir,fname,dir,ds,pstyle)  - use pstyle for tangent plotting
%
% See also cont, bifdetec
function q=swibra(dir,fname,varargin)
if(ischar(dir)) p=loadp(dir,fname);
else;  p=dir; if nargin>1; varargin{2}=fname; end; fname=inputname(1);
end
if p.sol.ptype==3; fprintf('\n Hopf-point, use hoswibra\n'); q=[]; return; end
if(p.sol.ptype~=1 && p.sw.spcont~=1)
    fprintf('\nInitialization error: not a branch point.\n'); %q=p; return;
end
u=p.u; lam=getlam(p); tau=p.tau; xi=p.sol.xi; lvar=length(varargin); 
pstyle=p.plot.pstyle; del=1e-3; % don't choose del too small!
if lvar>1; p.sol.ds=varargin{2}; gotds=1; lvar=lvar-1; 
    if lvar>1; pstyle=varargin{3}; end
else p.sol.ds=p.nc.dsmax/10; gotds=0; end 
r=resi(p,u); [Gu,Glam]=getder(p,u,r); % residual and jacobians 
% now calc al0, al1, phi0
u0d=tau(1:p.nu+p.nc.nq); al0=tau(p.nu+p.nc.nq+1); 
j=1; % use j=1 eigenval nearest to zero 
if p.sw.eigsstart==1; vs=size(Gu,1); % set eigs startvector
    p.sw.evopts.v0=ones(vs,1)/vs; end 
M=getM(p);  % now trivially extend M in case of auxiliary equations
if (p.nc.nq>0); M=[[M zeros(p.nu,p.nc.nq)]; [zeros(p.nc.nq,p.nu) 0*speye(p.nc.nq)]]; end  
%M=speye(p.nu+p.nc.nq); 
[phi1v,mu]=eigs(Gu,M,j,0,p.sw.evopts);
%[phi1v,mu]=eigs(Gu,j,0,p.sw.evopts); 11
phi1=real(phi1v(:,j));
if al0==0; fprintf('Degenerate branch point with d/ds lambda=0');return; end
fprintf('lambda=%g, zero eigenvalue is %g\n',lam,mu(j,j));
[psi1v,mu]=eigs(Gu',M,j,0,p.sw.evopts); psi1=psi1v(:,j);psi1=psi1/(phi1'*psi1); 
al1=psi1'*u0d;  phi0=(u0d-al1*phi1)/al0; 
rp=resi(p,au2u(p,u2au(p,u)+del*phi1)); % residual and jacobian at u+del*phi1 
[Gup,Glamp]=getder(p,au2u(p,u2au(p,u)+del*phi1),rp); Gud=Gup-Gu; 
if any(Gud) 
   Glamd=Glamp-Glam; a1=psi1'*(Gud*phi1)/del; 
   b1=psi1'*(Gud*phi0+Glamd)/del; 
   al1b=-(a1*al1/al0+2*b1); 
   fprintf('al0=%g, al1=%g, a1=%g, b1=%g, al1b=%g\n',al0,al1,a1,b1,al1b); 
   if al1b==0 fprintf('No distinct branch to switch to.');return; end
   tau1=[al1b*phi1+a1*phi0; a1];  
else % trivial branch 
    tau1=[phi1; 0];fprintf('trivial swibra\n');
end
tau1=tau1/xinorm(tau1,xi,p.nc.nq,p.sol.xiq); %tau1(end), pause
plotsolu(p,[-tau1;p.u(p.nu+1:end)],6,p.plot.pcmp,pstyle);
title(['\tau_' mat2str(p.plot.pcmp) ' at ' fname]); 
%xi=1/p.np; % the "natural choice"
if(p.sw.inter>1); p.sol.xi=asknu('xi',xi); 
    if ~gotds; p.sol.ds=asknu('ds',p.sol.ds); end 
end
p.tau=tau1; bs=p.branch(:,size(p.branch,2)); p=resetc(p); % set counters (note branch set below)
p.sol.ptype=-2; bs(1)=0; bs(2)=p.sol.ptype; % put last of p on new branch
if(~isempty(varargin{1})); [p,ok]=setfn(p,varargin{1}); if ok~=1; q=p; return; end
else fprintf('warning: problem directory unchanged.\n'); end
p.branch=bs; p.sol.deta=0; p.file.count=0; 
p.fuha.savefu(p); p.file.count=1; 
q=p;
